"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

import enum
from typing import TYPE_CHECKING, Any, Union

from ..models import ResourceGroup, ResourceType
from ..models.light import *  # noqa: F401, F403
from ..models.light.configuration_tool import ConfigurationTool
from ..models.light.cover import Cover
from ..models.light.fan import *  # noqa: F401, F403
from ..models.light.fan import Fan
from ..models.light.light import Light
from ..models.light.lock import Lock
from ..models.light.range_extender import RangeExtender
from ..models.light.siren import Siren
from .api import APIItems, GroupedAPIItems

if TYPE_CHECKING:
    from ..gateway import DeconzSession


class Alert(enum.Enum):
    """Temporary alert effect.

    "none" — light is not performing an alert.
    "lselect" — light is blinking a longer time.
    "select" — light is blinking a short time.
    """

    NONE = "none"
    LONG = "lselect"
    SHORT = "select"


class CoverAction(enum.Enum):
    """Possible cover actions."""

    CLOSE = enum.auto()
    OPEN = enum.auto()
    STOP = enum.auto()


class Effect(enum.Enum):
    """Effect of the light.

    "colorloop" — cycle through hue values 0-360.
    "none" — no effect.
    """

    COLORLOOP = "colorloop"
    NONE = "none"


class FanSpeed(enum.Enum):
    """Possible fan speeds."""

    OFF = 0
    PERCENT_25 = 1
    PERCENT_50 = 2
    PERCENT_75 = 3
    PERCENT_100 = 4
    AUTO = 5
    COMFORT_BREEZE = 6


class ConfigurationToolHandler(APIItems[ConfigurationTool]):
    """Handler for configuration tool."""

    resource_group = ResourceGroup.LIGHT
    resource_type = ResourceType.CONFIGURATION_TOOL
    item_cls = ConfigurationTool


class CoverHandler(APIItems[Cover]):
    """Handler for covers."""

    resource_group = ResourceGroup.LIGHT
    resource_types = {
        ResourceType.LEVEL_CONTROLLABLE_OUTPUT,
        ResourceType.WINDOW_COVERING_CONTROLLER,
        ResourceType.WINDOW_COVERING_DEVICE,
    }
    item_cls = Cover

    async def set_state(
        self,
        id: str,
        action: CoverAction | None = None,
        lift: int | None = None,
        tilt: int | None = None,
    ) -> dict[str, Any]:
        """Set state of cover.

        Action [CoverAction] Open, Close, Stop
        Lift [int] between 0-100.
        Tilt [int] between 0-100.
        """
        data: dict[str, bool | int] = {}

        if action is not None:
            if action is CoverAction.OPEN:
                data["open"] = True
            elif action is CoverAction.CLOSE:
                data["open"] = False
            elif action is CoverAction.STOP:
                data["stop"] = True
        else:
            if lift is not None:
                data["lift"] = lift
            if tilt is not None:
                data["tilt"] = tilt

        return await self.gateway.request(
            "put",
            path=f"{self.path}/{id}/state",
            json=data,
        )


class FanHandler(APIItems[Fan]):
    """Handler for locks."""

    resource_group = ResourceGroup.LIGHT
    resource_type = ResourceType.FAN
    item_cls = Fan

    async def set_state(self, id: str, speed: FanSpeed) -> dict[str, Any]:
        """Set speed of fans/ventilators.

        Speed [FanSpeed] Off, 25%, 50%, 75%, 100%, Auto, ComfortBreeze.
        """
        return await self.gateway.request(
            "put",
            path=f"{self.path}/{id}/state",
            json={"speed": speed.value},
        )


class LightHandler(APIItems[Light]):
    """Handler for lights."""

    resource_group = ResourceGroup.LIGHT
    resource_types = {
        ResourceType.COLOR_DIMMABLE_LIGHT,
        ResourceType.COLOR_LIGHT,
        ResourceType.COLOR_TEMPERATURE_LIGHT,
        ResourceType.EXTENDED_COLOR_LIGHT,
        ResourceType.DIMMABLE_LIGHT,
        ResourceType.DIMMABLE_PLUGIN_UNIT,
        ResourceType.ON_OFF_LIGHT,
        ResourceType.ON_OFF_OUTPUT,
        ResourceType.ON_OFF_PLUGIN_UNIT,
        ResourceType.SMART_PLUG,
        ResourceType.UNKNOWN,  # Legacy support
    }
    item_cls = Light

    async def set_state(
        self,
        id: str,
        alert: Alert | None = None,
        brightness: int | None = None,
        color_loop_speed: int | None = None,
        color_temperature: int | None = None,
        effect: Effect | None = None,
        hue: int | None = None,
        on: bool | None = None,
        on_time: int | None = None,
        saturation: int | None = None,
        transition_time: int | None = None,
        xy: tuple[float, float] | None = None,
    ) -> dict[str, Any]:
        """Change state of a light.

        Supported values:
        - alert [str]
          - "none" light is not performing an alert
          - "select" light is blinking a short time
          - "lselect" light is blinking a longer time
        - brightness [int] 0-255
        - color_loop_speed [int] 1-255
          - 1 = very fast
          - 15 is default
          - 255 very slow
        - color_temperature [int] between ctmin-ctmax
        - effect [str]
          - "none" no effect
          - "colorloop" the light will cycle continuously through all
                        colors with the speed specified by colorloopspeed
        - hue [int] 0-65535
        - on [bool] True/False
        - on_time [int] 0-65535 1/10 seconds resolution
        - saturation [int] 0-255
        - transition_time [int] 0-65535 1/10 seconds resolution
        - xy [tuple] (0-1, 0-1)
        """
        data: dict[str, Any] = {
            key: value
            for key, value in {
                "bri": brightness,
                "colorloopspeed": color_loop_speed,
                "ct": color_temperature,
                "hue": hue,
                "on": on,
                "ontime": on_time,
                "sat": saturation,
                "transitiontime": transition_time,
                "xy": xy,
            }.items()
            if value is not None
        }
        if alert is not None:
            data["alert"] = alert.value
        if effect is not None:
            data["effect"] = effect.value
        return await self.gateway.request(
            "put",
            path=f"{self.path}/{id}/state",
            json=data,
        )


class LockHandler(APIItems[Lock]):
    """Handler for fans."""

    resource_group = ResourceGroup.LIGHT
    resource_type = ResourceType.DOOR_LOCK
    item_cls = Lock

    async def set_state(self, id: str, lock: bool) -> dict[str, Any]:
        """Set state of lock.

        Lock [bool] True/False.
        """
        return await self.gateway.request(
            "put",
            path=f"{self.path}/{id}/state",
            json={"on": lock},
        )


class RangeExtenderHandler(APIItems[ConfigurationTool]):
    """Handler for range extender."""

    resource_group = ResourceGroup.LIGHT
    resource_type = ResourceType.RANGE_EXTENDER
    item_cls = RangeExtender


class SirenHandler(APIItems[Siren]):
    """Handler for sirens."""

    resource_group = ResourceGroup.LIGHT
    resource_type = ResourceType.WARNING_DEVICE
    item_cls = Siren

    async def set_state(
        self,
        id: str,
        on: bool,
        duration: int | None = None,
    ) -> dict[str, Any]:
        """Turn on device.

        Duration is counted as 1/10 of a second.
        """
        data: dict[str, int | str] = {}
        if on:
            data["alert"] = "lselect"
            if duration is not None:
                data["ontime"] = duration
        else:
            data["alert"] = "none"
        return await self.gateway.request(
            "put",
            path=f"{self.path}/{id}/state",
            json=data,
        )


LightResources = Union[
    ConfigurationTool,
    Cover,
    Fan,
    Light,
    Lock,
    Siren,
]


class LightResourceManager(GroupedAPIItems[LightResources]):
    """Represent deCONZ lights."""

    resource_group = ResourceGroup.LIGHT

    def __init__(self, gateway: DeconzSession) -> None:
        """Initialize light manager."""
        self.configuration_tool = ConfigurationToolHandler(gateway)
        self.covers = CoverHandler(gateway)
        self.fans = FanHandler(gateway)
        self.lights = LightHandler(gateway)
        self.locks = LockHandler(gateway)
        self.range_extender = RangeExtenderHandler(gateway)
        self.sirens = SirenHandler(gateway)

        handlers: list[APIItems[Any]] = [
            self.configuration_tool,
            self.covers,
            self.fans,
            self.lights,
            self.locks,
            self.range_extender,
            self.sirens,
        ]

        super().__init__(gateway, handlers)



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedBatteryState(TypedDict):
    """Battery state type definition."""

    battery: int


class TypedBattery(TypedDict):
    """Battery type definition."""

    state: TypedBatteryState


class Battery(SensorBase):
    """Battery sensor."""

    ZHATYPE = ("ZHABattery",)

    raw: TypedBattery

    @property
    def battery(self) -> int:
        """Battery."""
        return self.raw["state"]["battery"]



"""API base classes."""

from __future__ import annotations

from collections.abc import Callable, ItemsView, ValuesView
from typing import TYPE_CHECKING, Any, Generic, Iterator, KeysView, Optional

from ..models import DataResource, ResourceGroup, ResourceType
from ..models.event import Event, EventType

if TYPE_CHECKING:
    from ..gateway import DeconzSession


SubscriptionType = tuple[
    Callable[[EventType, str], None],
    Optional[tuple[EventType, ...]],
]
UnsubscribeType = Callable[[], None]


class APIItems(Generic[DataResource]):
    """Base class for a map of API Items."""

    resource_group: ResourceGroup
    resource_type = ResourceType.UNKNOWN
    resource_types: set[ResourceType] | None = None
    item_cls: Any

    def __init__(self, gateway: DeconzSession) -> None:
        """Initialize API items."""
        self.gateway = gateway
        self._items: dict[str, DataResource] = {}
        self._subscribers: list[SubscriptionType] = []

        self.path = f"/{self.resource_group.value}"

        if self.resource_types is None:
            self.resource_types = {self.resource_type}

    def post_init(self) -> None:
        """Post initialization method."""
        self.gateway.events.subscribe(
            self.process_event,
            event_filter=(EventType.ADDED, EventType.CHANGED),
            resource_filter=self.resource_group,
        )

    async def update(self) -> None:
        """Refresh data."""
        raw = await self.gateway.request("get", f"/{self.resource_group.value}")
        self.process_raw(raw)

    def process_raw(self, raw: dict[str, dict[str, Any]]) -> None:
        """Process full data."""
        for id, raw_item in raw.items():
            self.process_item(id, raw_item)

    def process_event(self, event: Event) -> None:
        """Process event."""
        if event.type == EventType.CHANGED and event.id in self:
            self.process_item(event.id, event.changed_data)
            return

        if event.type == EventType.ADDED and event.id not in self:
            self.process_item(event.id, event.added_data)

    def process_item(self, id: str, raw: dict[str, Any]) -> None:
        """Process data."""
        if id in self._items:
            obj = self._items[id]
            obj.update(raw)
            event = EventType.CHANGED

        else:
            self._items[id] = self.item_cls(id, raw, self.gateway.request)
            event = EventType.ADDED

        for callback, event_filter in self._subscribers:
            if event_filter is not None and event not in event_filter:
                continue
            callback(event, id)

    def subscribe(
        self,
        callback: Callable[[EventType, str], None],
        event_filter: tuple[EventType, ...] | EventType | None = None,
    ) -> UnsubscribeType:
        """Subscribe to events.

        "callback" - callback function to call when on event.
        Return function to unsubscribe.
        """
        if isinstance(event_filter, EventType):
            event_filter = (event_filter,)

        subscription = (callback, event_filter)
        self._subscribers.append(subscription)

        def unsubscribe() -> None:
            self._subscribers.remove(subscription)

        return unsubscribe

    def items(self) -> ItemsView[str, DataResource]:
        """Return items."""
        return self._items.items()

    def keys(self) -> KeysView[str]:
        """Return item keys."""
        return self._items.keys()

    def values(self) -> ValuesView[DataResource]:
        """Return item values."""
        return self._items.values()

    def __getitem__(self, obj_id: str) -> DataResource:
        """Get item value based on key."""
        return self._items[obj_id]

    def __iter__(self) -> Iterator[str]:
        """Allow iterate over items."""
        return iter(self._items)


class GroupedAPIItems(Generic[DataResource]):
    """Represent a group of deCONZ API items."""

    resource_group: ResourceGroup

    def __init__(
        self, gateway: DeconzSession, api_items: list[APIItems[DataResource]]
    ) -> None:
        """Initialize sensor manager."""
        self.gateway = gateway
        self._items = api_items

        self._type_to_handler: dict[ResourceType, APIItems[DataResource]] = {
            resource_type: handler
            for handler in api_items
            if handler.resource_types is not None
            for resource_type in handler.resource_types
        }

    def post_init(self) -> None:
        """Post initialization method."""
        self.gateway.events.subscribe(
            self.process_event,
            event_filter=(EventType.ADDED, EventType.CHANGED),
            resource_filter=self.resource_group,
        )

    def process_raw(self, raw: dict[str, dict[str, Any]]) -> None:
        """Process full data."""
        for id, raw_item in raw.items():
            self.process_item(id, raw_item)

    def process_event(self, event: Event) -> None:
        """Process event."""
        if event.type == EventType.CHANGED and event.id in self:
            self.process_item(event.id, event.changed_data)
            return

        if event.type == EventType.ADDED and event.id not in self:
            self.process_item(event.id, event.added_data)

    def process_item(self, id: str, raw: dict[str, Any]) -> None:
        """Process item data."""
        if obj := self.get(id):
            obj.update(raw)
            return

        handler = self._type_to_handler[ResourceType(raw.get("type"))]
        handler.process_item(id, raw)

    def subscribe(
        self,
        callback: Callable[[EventType, str], None],
        event_filter: tuple[EventType, ...] | EventType | None = None,
    ) -> UnsubscribeType:
        """Subscribe to state changes for all grouped resources."""
        subscribers = [x.subscribe(callback, event_filter) for x in self._items]

        def unsubscribe() -> None:
            for subscriber in subscribers:
                subscriber()

        return unsubscribe

    def items(self) -> dict[str, DataResource]:
        """Return items."""
        return {k: v for i in self._items for k, v in i.items()}

    def keys(self) -> list[str]:
        """Return item keys."""
        return [k for i in self._items for k in i]

    def values(self) -> list[DataResource]:
        """Return item values."""
        return [v for i in self._items for v in i.values()]

    def get(self, id: str, default: Any = None) -> DataResource | None:
        """Get item value based on key, if no match return default."""
        return next((i[id] for i in self._items if id in i), default)

    def __getitem__(self, obj_id: str) -> DataResource:
        """Get item value based on key."""
        return self.items()[obj_id]

    def __iter__(self) -> Iterator[str]:
        """Allow iterate over items."""
        return iter(self.items())



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Final, Literal, TypedDict

from . import SensorBase

# Action and Panel
ANCILLARY_CONTROL_ARMED_AWAY: Final = "armed_away"
ANCILLARY_CONTROL_ARMED_NIGHT: Final = "armed_night"
ANCILLARY_CONTROL_ARMED_STAY: Final = "armed_stay"
ANCILLARY_CONTROL_DISARMED: Final = "disarmed"

# Action only
ANCILLARY_CONTROL_EMERGENCY: Final = "emergency"
ANCILLARY_CONTROL_FIRE: Final = "fire"
ANCILLARY_CONTROL_INVALID_CODE: Final = "invalid_code"
ANCILLARY_CONTROL_PANIC: Final = "panic"

# Panel only
ANCILLARY_CONTROL_ARMING_AWAY: Final = "arming_away"
ANCILLARY_CONTROL_ARMING_NIGHT: Final = "arming_night"
ANCILLARY_CONTROL_ARMING_STAY: Final = "arming_stay"
ANCILLARY_CONTROL_ENTRY_DELAY: Final = "entry_delay"
ANCILLARY_CONTROL_EXIT_DELAY: Final = "exit_delay"
ANCILLARY_CONTROL_IN_ALARM: Final = "in_alarm"
ANCILLARY_CONTROL_NOT_READY: Final = "not_ready"


class TypedAncillaryControlState(TypedDict):
    """Ancillary control state type definition."""

    action: Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "disarmed",
        "emergency",
        "fire",
        "invalid_code",
        "panic",
    ]
    panel: Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "arming_away",
        "arming_night",
        "arming_stay",
        "disarmed",
        "entry_delay",
        "exit_delay",
        "in_alarm",
        "not_ready",
    ]
    seconds_remaining: int


class TypedAncillaryControl(TypedDict):
    """Ancillary control type definition."""

    state: TypedAncillaryControlState


class AncillaryControl(SensorBase):
    """Ancillary control sensor."""

    ZHATYPE = ("ZHAAncillaryControl",)

    raw: TypedAncillaryControl

    @property
    def action(
        self,
    ) -> Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "disarmed",
        "emergency",
        "fire",
        "invalid_code",
        "panic",
    ]:
        """Last action a user invoked on the keypad.

        Supported values:
        - "armed_away"
        - "armed_night"
        - "armed_stay"
        - "disarmed"
        - "emergency"
        - "fire"
        - "invalid_code"
        - "panic"
        """
        return self.raw["state"]["action"]

    @property
    def panel(
        self,
    ) -> Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "arming_away",
        "arming_night",
        "arming_stay",
        "disarmed",
        "entry_delay",
        "exit_delay",
        "in_alarm",
        "not_ready",
    ] | None:
        """Mirror of alarm system state.armstate attribute.

        It reflects what is shown on the panel (when activated by the keypad’s proximity sensor).

        Supported values:
        - "armed_away"
        - "armed_night"
        - "armed_stay"
        - "arming_away"
        - "arming_night"
        - "arming_stay"
        - "disarmed"
        - "entry_delay"
        - "exit_delay"
        - "in_alarm"
        - "not_ready"
        """
        return self.raw["state"].get("panel")

    @property
    def seconds_remaining(self) -> int:
        """Remaining time of "exit_delay" and "entry_delay" states.

        In all other states the value is 0.
        """
        return self.raw["state"].get("seconds_remaining", 0)



"""Python library to connect deCONZ and Home Assistant to work together."""

from . import LightBase


class RangeExtender(LightBase):
    """ZigBee range extender."""

    ZHATYPE = ("Range extender",)



"""Python library to connect deCONZ and Home Assistant to work together."""

from . import LightBase


class ConfigurationTool(LightBase):
    """deCONZ hardware antenna."""

    ZHATYPE = ("Configuration tool",)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import TypedDict

from . import SensorBase


class TypedVibrationConfig(TypedDict):
    """Vibration config type definition."""

    sensitivity: int
    sensitivitymax: int


class TypedVibrationState(TypedDict):
    """Vibration state type definition."""

    orientation: list[str]
    tiltangle: int
    vibration: bool
    vibrationstrength: int


class TypedVibration(TypedDict):
    """Vibration type definition."""

    config: TypedVibrationConfig
    state: TypedVibrationState


class Vibration(SensorBase):
    """Vibration sensor."""

    ZHATYPE = ("ZHAVibration",)

    raw: TypedVibration

    @property
    def orientation(self) -> list[str] | None:
        """Orientation."""
        return self.raw["state"].get("orientation")

    @property
    def sensitivity(self) -> int | None:
        """Vibration sensitivity."""
        return self.raw["config"].get("sensitivity")

    @property
    def max_sensitivity(self) -> int | None:
        """Vibration max sensitivity."""
        return self.raw["config"].get("sensitivitymax")

    @property
    def tilt_angle(self) -> int | None:
        """Tilt angle."""
        return self.raw["state"].get("tiltangle")

    @property
    def vibration(self) -> bool:
        """Vibration."""
        return self.raw["state"]["vibration"]

    @property
    def vibration_strength(self) -> int | None:
        """Strength of vibration."""
        return self.raw["state"].get("vibrationstrength")



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any

from ..models import ResourceGroup
from ..models.event import EventType
from ..models.scene import Scene
from .api import APIItems


class Scenes(APIItems[Scene]):
    """Represent scenes of a deCONZ group."""

    item_cls = Scene
    resource_group = ResourceGroup.SCENE

    def post_init(self) -> None:
        """Register for group data events."""
        self.gateway.groups.subscribe(
            self.group_data_callback,
            event_filter=(EventType.ADDED, EventType.CHANGED),
        )

    async def create_scene(self, group_id: str, name: str) -> dict[str, Any]:
        """Create a new scene.

        The current state of each light will become the lights scene state.
        """
        return await self.gateway.request(
            "post",
            path=f"/groups/{group_id}/scenes",
            json={"name": name},
        )

    async def recall(self, group_id: str, scene_id: str) -> dict[str, Any]:
        """Recall scene to group."""
        return await self.gateway.request(
            "put",
            path=f"/groups/{group_id}/scenes/{scene_id}/recall",
            json={},
        )

    async def store(self, group_id: str, scene_id: str) -> dict[str, Any]:
        """Store current group state in scene.

        The actual state of each light in the group will become the lights scene state.
        """
        return await self.gateway.request(
            "put",
            path=f"/groups/{group_id}/scenes/{scene_id}/store",
            json={},
        )

    async def set_attributes(
        self, group_id: str, scene_id: str, name: str | None = None
    ) -> dict[str, Any]:
        """Change attributes of scene.

        Supported values:
        - name [str]
        """
        data = {
            key: value
            for key, value in {
                "name": name,
            }.items()
            if value is not None
        }
        return await self.gateway.request(
            "put",
            path=f"/groups/{group_id}/scenes/{scene_id}",
            json=data,
        )

    def group_data_callback(self, action: EventType, group_id: str) -> None:
        """Subscribe callback for new group data."""
        self.process_item(group_id, {})

    def process_item(self, id: str, raw: dict[str, Any]) -> None:
        """Pre-process scene data."""
        group = self.gateway.groups[id]

        for scene in group.raw["scenes"]:
            super().process_item(
                f'{id}_{scene["id"]}',
                scene | {"group_id": id, "group_name": group.name},
            )



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Final, Literal, TypedDict

from . import SensorBase

THERMOSTAT_MODE_AUTO: Final = "auto"
THERMOSTAT_MODE_COOL: Final = "cool"
THERMOSTAT_MODE_DRY: Final = "dry"
THERMOSTAT_MODE_FAN_ONLY: Final = "fan only"
THERMOSTAT_MODE_HEAT: Final = "heat"
THERMOSTAT_MODE_EMERGENCY_HEATING: Final = "emergency heating"
THERMOSTAT_MODE_OFF: Final = "off"
THERMOSTAT_MODE_PRECOOLING: Final = "precooling"
THERMOSTAT_MODE_SLEEP: Final = "sleep"

THERMOSTAT_FAN_MODE_AUTO: Final = "auto"
THERMOSTAT_FAN_MODE_HIGH: Final = "high"
THERMOSTAT_FAN_MODE_LOW: Final = "low"
THERMOSTAT_FAN_MODE_MEDIUM: Final = "medium"
THERMOSTAT_FAN_MODE_OFF: Final = "off"
THERMOSTAT_FAN_MODE_ON: Final = "on"
THERMOSTAT_FAN_MODE_SMART: Final = "smart"

THERMOSTAT_PRESET_AUTO: Final = "auto"
THERMOSTAT_PRESET_BOOST: Final = "boost"
THERMOSTAT_PRESET_COMFORT: Final = "comfort"
THERMOSTAT_PRESET_COMPLEX: Final = "complex"
THERMOSTAT_PRESET_ECO: Final = "eco"
THERMOSTAT_PRESET_HOLIDAY: Final = "holiday"
THERMOSTAT_PRESET_MANUAL: Final = "manual"

THERMOSTAT_SWING_MODE_FULLY_CLOSED: Final = "fully closed"
THERMOSTAT_SWING_MODE_FULLY_OPEN: Final = "fully open"
THERMOSTAT_SWING_MODE_HALF_OPEN: Final = "half open"
THERMOSTAT_SWING_MODE_QUARTER_OPEN: Final = "quarter open"
THERMOSTAT_SWING_MODE_THREE_QUARTERS_OPEN: Final = "three quarters open"

THERMOSTAT_TEMPERATURE_MEASUREMENT_MODE_AIR_SENSOR: Final = "air sensor"
THERMOSTAT_TEMPERATURE_MEASUREMENT_MODE_FLOOR_PROTECTION: Final = "floor protection"
THERMOSTAT_TEMPERATURE_MEASUREMENT_MODE_FLOOR_SENSOR: Final = "floor sensor"


class TypedThermostatConfig(TypedDict):
    """Thermostat config type definition."""

    coolsetpoint: int
    displayflipped: bool
    externalsensortemp: int
    externalwindowopen: bool
    fanmode: Literal["off", "low", "medium", "high", "on", "auto", "smart"]
    heatsetpoint: int
    locked: bool
    mode: Literal[
        "off",
        "auto",
        "cool",
        "heat",
        "emergency heating",
        "precooling",
        "fan only",
        "dry",
        "sleep",
    ]
    mountingmode: bool
    offset: int
    preset: Literal["holiday", "auto", "manual", "comfort", "eco", "boost", "complex"]
    schedule_on: bool
    swingmode: Literal[
        "fully closed", "fully open", "quarter open", "half open", "three quarters open"
    ]
    temperaturemeasurement: Literal["air sensor", "floor sensor", "floor protection"]
    windowopen_set: bool


class TypedThermostatState(TypedDict):
    """Thermostat state type definition."""

    errorcode: bool
    floortemperature: int
    heating: bool
    mountingmodeactive: bool
    on: bool
    temperature: int
    valve: int


class TypedThermostat(TypedDict):
    """Thermostat type definition."""

    config: TypedThermostatConfig
    state: TypedThermostatState


class Thermostat(SensorBase):
    """Thermostat "sensor"."""

    ZHATYPE = ("ZHAThermostat", "CLIPThermostat")

    raw: TypedThermostat

    @property
    def cooling_setpoint(self) -> float | None:
        """Cooling setpoint.

        700-3500.
        """
        if temperature := self.raw["config"].get("coolsetpoint"):
            return round(temperature / 100, 1)
        return None

    @property
    def display_flipped(self) -> bool | None:
        """Tells if display for TRVs is flipped."""
        return self.raw["config"].get("displayflipped")

    @property
    def error_code(self) -> bool | None:
        """Error code."""
        return self.raw["state"].get("errorcode")

    @property
    def external_sensor_temperature(self) -> float | None:
        """Track temperature value provided by an external sensor.

        -32768–32767.
        Modes are device dependent and only exposed for devices supporting it.
        """
        if temperature := self.raw["config"].get("externalsensortemp"):
            return round(temperature / 100, 1)
        return None

    @property
    def external_window_open(self) -> bool | None:
        """Track open/close state of an external sensor.

        Modes are device dependent and only exposed for devices supporting it.
        """
        return self.raw["config"].get("externalwindowopen")

    @property
    def fan_mode(
        self,
    ) -> Literal["off", "low", "medium", "high", "on", "auto", "smart"] | None:
        """Fan mode.

        Supported values:
        - "off"
        - "low"
        - "medium"
        - "high"
        - "on"
        - "auto"
        - "smart"
        Modes are device dependent and only exposed for devices supporting it.
        """
        return self.raw["config"].get("fanmode")

    @property
    def floor_temperature(self) -> float | None:
        """Floor temperature."""
        if temperature := self.raw["state"].get("floortemperature"):
            return round(temperature / 100, 1)
        return None

    @property
    def heating(self) -> bool | None:
        """Heating setpoint."""
        return self.raw["state"].get("heating")

    @property
    def heating_setpoint(self) -> float | None:
        """Heating setpoint.

        500-3200.
        """
        if temperature := self.raw["config"].get("heatsetpoint"):
            return round(temperature / 100, 1)
        return None

    @property
    def locked(self) -> bool | None:
        """Child lock active/inactive for thermostats/TRVs supporting it."""
        return self.raw["config"].get("locked")

    @property
    def mode(
        self,
    ) -> Literal[
        "off",
        "auto",
        "cool",
        "heat",
        "emergency heating",
        "precooling",
        "fan only",
        "dry",
        "sleep",
    ] | None:
        """Set the current operating mode of a thermostat.

        Supported values:
        - "off"
        - "auto"
        - "cool"
        - "heat"
        - "emergency heating"
        - "precooling"
        - "fan only"
        - "dry"
        - "sleep"
        Modes are device dependent and only exposed for devices supporting it.
        """
        return self.raw["config"].get("mode")

    @property
    def mounting_mode(self) -> bool | None:
        """Set a TRV into mounting mode if supported (valve fully open position)."""
        return self.raw["config"].get("mountingmode")

    @property
    def mounting_mode_active(self) -> bool | None:
        """If thermostat mounting mode is active."""
        return self.raw["state"].get("mountingmodeactive")

    @property
    def offset(self) -> int | None:
        """Add a signed offset value to measured temperature and humidity state values. Values send by the REST-API are already amended by the offset."""
        return self.raw["config"].get("offset")

    @property
    def preset(
        self,
    ) -> Literal[
        "holiday", "auto", "manual", "comfort", "eco", "boost", "complex"
    ] | None:
        """Set the current operating mode for Tuya thermostats.

        Supported values:
        - "holiday"
        - "auto"
        - "manual"
        - "comfort"
        - "eco"
        - "boost"
        - "complex"
        Modes are device dependent and only exposed for devices supporting it.
        """
        return self.raw["config"].get("preset")

    @property
    def schedule_enabled(self) -> bool | None:
        """Tell when thermostat schedule is enabled."""
        return self.raw["config"].get("schedule_on")

    @property
    def state_on(self) -> bool | None:
        """Declare if the sensor is on or off."""
        return self.raw["state"].get("on")

    @property
    def swing_mode(
        self,
    ) -> Literal[
        "fully closed",
        "fully open",
        "quarter open",
        "half open",
        "three quarters open",
    ] | None:
        """Set the AC louvers position.

        Supported values:
        - "fully closed"
        - "fully open"
        - "quarter open"
        - "half open"
        - "three quarters open"
        Modes are device dependent and only exposed for devices supporting it.
        """
        return self.raw["config"].get("swingmode")

    @property
    def temperature(self) -> int:
        """Temperature."""
        return self.raw["state"]["temperature"]

    @property
    def scaled_temperature(self) -> float:
        """Scaled temperature."""
        return round(self.temperature / 100, 1)

    @property
    def temperature_measurement(
        self,
    ) -> Literal["air sensor", "floor sensor", "floor protection"] | None:
        """Set the mode of operation for Elko Super TR thermostat.

        Supported values:
        - "air sensor"
        - "floor sensor"
        - "floor protection"
        """
        return self.raw["config"].get("temperaturemeasurement")

    @property
    def valve(self) -> int | None:
        """How open the valve is."""
        return self.raw["state"].get("valve")

    @property
    def window_open_detection(self) -> bool | None:
        """Set if window open detection shall be active or inactive for Tuya thermostats.

        (Support is device dependent).
        """
        return self.raw["config"].get("windowopen_set")

    async def set_config(
        self,
        cooling_setpoint: int | None = None,
        enable_schedule: bool | None = None,
        external_sensor_temperature: int | None = None,
        external_window_open: bool | None = None,
        fan_mode: Literal["off", "low", "medium", "high", "on", "auto", "smart"]
        | str
        | None = None,
        flip_display: bool | None = None,
        heating_setpoint: int | None = None,
        locked: bool | None = None,
        mode: Literal[
            "off",
            "auto",
            "cool",
            "heat",
            "emergency heating",
            "precooling",
            "fan only",
            "dry",
            "sleep",
        ]
        | str
        | None = None,
        mounting_mode: bool | None = None,
        on: bool | None = None,
        preset: Literal[
            "holiday", "auto", "manual", "comfort", "eco", "boost", "complex"
        ]
        | str
        | None = None,
        schedule: list[str] | None = None,
        set_valve: bool | None = None,
        swing_mode: Literal[
            "fully closed",
            "fully open",
            "quarter open",
            "half open",
            "three quarters open",
        ]
        | str
        | None = None,
        temperature_measurement: Literal[
            "air sensor", "floor sensor", "floor protection"
        ]
        | str
        | None = None,
        window_open_detection: bool | None = None,
    ) -> dict[str, Any]:
        """Change config of thermostat.

        Supported values:
        - cooling_setpoint [int] 700-3500
        - enable_schedule [bool] True/False
        - external_sensor_temperature [int] -32768-32767
        - external_window_open [bool] True/False
        - fan_mode [str]
          - "auto"
          - "high"
          - "low"
          - "medium"
          - "off"
          - "on"
          - "smart"
        - flip_display [bool] True/False
        - heating_setpoint [int] 500-3200
        - locked [bool] True/False
        - mode [str]
          - "auto"
          - "cool"
          - "dry"
          - "emergency heating"
          - "fan only"
          - "heat"
          - "off"
          - "precooling"
          - "sleep"
        - mounting_mode [bool] True/False
        - on [bool] True/False
        - preset [str]
          - "auto"
          - "boost"
          - "comfort"
          - "complex"
          - "eco"
          - "holiday"
          - "manual"
        - schedule [list]
        - set_valve [bool] True/False
        - swing_mode [str]
          - "fully closed"
          - "fully open"
          - "half open"
          - "quarter open"
          - "three quarters open"
        - temperature_measurement [str]
          - "air sensor"
          - "floor protection"
          - "floor sensor"
        - window_open_detection [bool] True/False
        """
        data = {
            key: value
            for key, value in {
                "coolsetpoint": cooling_setpoint,
                "schedule_on": enable_schedule,
                "externalsensortemp": external_sensor_temperature,
                "externalwindowopen": external_window_open,
                "fanmode": fan_mode,
                "displayflipped": flip_display,
                "heatsetpoint": heating_setpoint,
                "locked": locked,
                "mode": mode,
                "mountingmode": mounting_mode,
                "on": on,
                "preset": preset,
                "schedule": schedule,
                "setvalve": set_valve,
                "swingmode": swing_mode,
                "temperaturemeasurement": temperature_measurement,
                "windowopen_set": window_open_detection,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/config", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedOpenCloseState(TypedDict):
    """Open close state type definition."""

    open: bool


class TypedOpenClose(TypedDict):
    """Open close type definition."""

    state: TypedOpenCloseState


class OpenClose(SensorBase):
    """Door/Window sensor."""

    ZHATYPE = ("ZHAOpenClose", "CLIPOpenClose")

    raw: TypedOpenClose

    @property
    def open(self) -> bool:
        """Door open."""
        return self.raw["state"]["open"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedGenericFlagState(TypedDict):
    """Generic flag state type definition."""

    flag: bool


class TypedGenericFlag(TypedDict):
    """Generic flag type definition."""

    state: TypedGenericFlagState


class GenericFlag(SensorBase):
    """Generic flag sensor."""

    ZHATYPE = ("CLIPGenericFlag",)

    raw: TypedGenericFlag

    @property
    def flag(self) -> bool:
        """Flag status."""
        return self.raw["state"]["flag"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedGenericStatusState(TypedDict):
    """Generic status state type definition."""

    status: str


class TypedGenericStatus(TypedDict):
    """Generic status type definition."""

    state: TypedGenericStatusState


class GenericStatus(SensorBase):
    """Generic status sensor."""

    ZHATYPE = ("CLIPGenericStatus",)

    raw: TypedGenericStatus

    @property
    def status(self) -> str:
        """Status."""
        return self.raw["state"]["status"]



"""Event data from deCONZ websocket."""

from __future__ import annotations

from dataclasses import dataclass
import enum
from typing import Any

from . import ResourceGroup


class EventKey(enum.Enum):
    """Event keys."""

    TYPE = "t"
    EVENT = "e"
    RESOURCE = "r"
    ID = "id"

    GROUP_ID = "gid"
    SCENE_ID = "scid"

    ATTRIBUTE = "attr"
    CONFIG = "config"
    NAME = "name"
    STATE = "state"
    UNIQUE_ID = "uniqueid"

    ALARM = "alarmsystem"
    GROUP = "group"
    LIGHT = "light"
    SENSOR = "sensor"


class EventType(enum.Enum):
    """The event type of the message."""

    ADDED = "added"
    CHANGED = "changed"
    DELETED = "deleted"
    SCENE_CALLED = "scene-called"


@dataclass
class Event:
    """Event data from deCONZ websocket."""

    id: str
    data: dict[str, Any]
    resource: ResourceGroup
    type: EventType

    # Only for "scene-called" events
    group_id: str
    scene_id: str

    @property
    def added_data(self) -> dict[str, Any]:
        """Full device resource.

        Only for "added" events.
        """
        data: dict[str, Any] = {}

        for key in (
            EventKey.SENSOR.value,
            EventKey.LIGHT.value,
            EventKey.ALARM.value,
            EventKey.GROUP.value,
        ):
            if key in self.data:
                data = self.data[key]
                break

        return data

    @property
    def changed_data(self) -> dict[str, Any]:
        """Altered device data.

        Only for "changed" events.
        """
        data: dict[str, Any] = {}

        for key in (
            EventKey.STATE.value,
            EventKey.CONFIG.value,
            EventKey.NAME.value,
        ):
            if (value := self.data.get(key)) is not None:
                data[key] = value

        return data

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "Event":
        """Create event instance from dict."""
        return cls(
            id=data.get(EventKey.ID.value, ""),
            group_id=data.get(EventKey.GROUP_ID.value, ""),
            scene_id=data.get(EventKey.SCENE_ID.value, ""),
            resource=ResourceGroup(data[EventKey.RESOURCE.value]),
            type=EventType(data[EventKey.EVENT.value]),
            data=data,
        )



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Final, Literal, TypedDict, cast

from .light import Light

FAN_SPEED_OFF: Final = 0
FAN_SPEED_25_PERCENT: Final = 1
FAN_SPEED_50_PERCENT: Final = 2
FAN_SPEED_75_PERCENT: Final = 3
FAN_SPEED_100_PERCENT: Final = 4
FAN_SPEED_AUTO: Final = 5
FAN_SPEED_COMFORT_BREEZE: Final = 6


class TypedFanState(TypedDict):
    """Fan state type definition."""

    speed: Literal[0, 1, 2, 3, 4, 5, 6]


class TypedFan(TypedDict):
    """Fan type definition."""

    state: TypedFanState


class Fan(Light):
    """Light fixture with fan control.

    0 - fan is off
    1 - 25%
    2 - 50%
    3 - 75%
    4 - 100%
    5 - Auto
    6 - "comfort-breeze"
    """

    ZHATYPE = ("Fan",)

    # raw: TypedFan

    def post_init(self) -> None:
        """Post init method."""
        self._raw = cast(TypedFan, self.raw)

    @property
    def speed(self) -> Literal[0, 1, 2, 3, 4, 5, 6]:
        """Speed of the fan."""
        return self._raw["state"]["speed"]

    async def set_speed(self, speed: Literal[0, 1, 2, 3, 4, 5, 6]) -> dict[str, Any]:
        """Set speed of fans/ventilators.

        Speed [int] between 0-6.
        """
        return await self.request(
            field=f"{self.deconz_id}/state",
            data={"speed": speed},
        )



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, TypedDict

from . import SensorBase


class TypedLightLevelConfig(TypedDict):
    """Light level config type definition."""

    tholddark: int
    tholdoffset: int


class TypedLightLevelState(TypedDict):
    """Light level state type definition."""

    dark: bool
    daylight: bool
    lightlevel: int
    lux: int


class TypedLightLevel(TypedDict):
    """Light level type definition."""

    config: TypedLightLevelConfig
    state: TypedLightLevelState


class LightLevel(SensorBase):
    """Light level sensor."""

    ZHATYPE = ("ZHALightLevel", "CLIPLightLevel")

    raw: TypedLightLevel

    @property
    def dark(self) -> bool | None:
        """If the area near the sensor is light or not."""
        return self.raw["state"].get("dark")

    @property
    def daylight(self) -> bool | None:
        """Daylight."""
        return self.raw["state"].get("daylight")

    @property
    def light_level(self) -> int:
        """Light level."""
        return self.raw["state"]["lightlevel"]

    @property
    def scaled_light_level(self) -> float:
        """Scaled light level."""
        return round(10 ** ((self.light_level - 1) / 10000), 1)

    @property
    def lux(self) -> int | None:
        """Lux."""
        return self.raw["state"].get("lux")

    @property
    def threshold_dark(self) -> int | None:
        """Threshold to hold dark."""
        return self.raw["config"].get("tholddark")

    @property
    def threshold_offset(self) -> int | None:
        """Offset for threshold to hold dark."""
        return self.raw["config"].get("tholdoffset")

    async def set_config(
        self,
        threshold_dark: int | None = None,
        threshold_offset: int | None = None,
    ) -> dict[str, Any]:
        """Change config of presence sensor.

        Supported values:
        - threshold_dark [int] 0-65534
        - threshold_offset [int] 1-65534
        """
        data = {
            key: value
            for key, value in {
                "tholddark": threshold_dark,
                "tholdoffset": threshold_offset,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/config", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, TypedDict

from . import SensorBase


class TypedHumidityConfig(TypedDict):
    """Humidity config type definition."""

    offset: int


class TypedHumidityState(TypedDict):
    """Humidity state type definition."""

    humidity: int


class TypedHumidity(TypedDict):
    """Humidity type definition."""

    config: TypedHumidityConfig
    state: TypedHumidityState


class Humidity(SensorBase):
    """Humidity sensor."""

    ZHATYPE = ("ZHAHumidity", "CLIPHumidity")

    raw: TypedHumidity

    @property
    def humidity(self) -> int:
        """Humidity level."""
        return self.raw["state"]["humidity"]

    @property
    def scaled_humidity(self) -> float:
        """Scaled humidity level."""
        return round(self.humidity / 100, 1)

    @property
    def offset(self) -> int | None:
        """Signed offset value to measured state values.

        Values send by the REST-API are already amended by the offset.
        """
        return self.raw["config"].get("offset")

    async def set_config(
        self,
        offset: int | None = None,
    ) -> dict[str, Any]:
        """Change config of humidity sensor.

        Supported values:
        - offset [int] -32768–32767
        """
        data = {
            key: value
            for key, value in {
                "offset": offset,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/config", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedWaterState(TypedDict):
    """Water state type definition."""

    water: bool


class TypedWater(TypedDict):
    """Water type definition."""

    state: TypedWaterState


class Water(SensorBase):
    """Water sensor."""

    ZHATYPE = ("ZHAWater",)

    raw: TypedWater

    @property
    def water(self) -> bool:
        """Water detected."""
        return self.raw["state"]["water"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedCarbonMonoxideState(TypedDict):
    """Carbon monoxide state type definition."""

    carbonmonoxide: bool


class TypedCarbonMonoxide(TypedDict):
    """Carbon monoxide type definition."""

    state: TypedCarbonMonoxideState


class CarbonMonoxide(SensorBase):
    """Carbon monoxide sensor."""

    ZHATYPE = ("ZHACarbonMonoxide",)

    raw: TypedCarbonMonoxide

    @property
    def carbon_monoxide(self) -> bool:
        """Carbon monoxide detected."""
        return self.raw["state"]["carbonmonoxide"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Final, Literal, TypedDict

from . import SensorBase

SWITCH_DEVICE_MODE_DUAL_PUSH_BUTTON: Final = "dualpushbutton"
SWITCH_DEVICE_MODE_DUAL_ROCKER: Final = "dualrocker"
SWITCH_DEVICE_MODE_SINGLE_PUSH_BUTTON: Final = "singlepushbutton"
SWITCH_DEVICE_MODE_SINGLE_ROCKER: Final = "singlerocker"

SWITCH_MODE_MOMENTARY: Final = "momentary"
SWITCH_MODE_ROCKER: Final = "rocker"


class TypedSwitchConfig(TypedDict):
    """Switch config type definition."""

    devicemode: Literal[
        "dualpushbutton", "dualrocker", "singlepushbutton", "singlerocker"
    ]
    mode: Literal["momentary", "rocker"]
    windowcoveringtype: Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9]


class TypedSwitchState(TypedDict):
    """Switch state type definition."""

    angle: int
    buttonevent: int
    eventduration: int
    gesture: int
    xy: tuple[float, float]


class TypedSwitch(TypedDict):
    """Switch type definition."""

    config: TypedSwitchConfig
    state: TypedSwitchState


class Switch(SensorBase):
    """Switch sensor."""

    ZHATYPE = ("ZHASwitch", "ZGPSwitch", "CLIPSwitch")

    raw: TypedSwitch

    @property
    def button_event(self) -> int | None:
        """Button press."""
        return self.raw["state"].get("buttonevent")

    @property
    def gesture(self) -> int | None:
        """Gesture used for Xiaomi magic cube."""
        return self.raw["state"].get("gesture")

    @property
    def angle(self) -> int | None:
        """Angle representing color on a tint remote color wheel."""
        return self.raw["state"].get("angle")

    @property
    def xy(self) -> tuple[float, float] | None:
        """X/Y color coordinates selected on a tint remote color wheel."""
        return self.raw["state"].get("xy")

    @property
    def event_duration(self) -> int | None:
        """Duration of a push button event for the Hue wall switch module.

        Increased with 8 for each x001, and they are issued pretty much every 800ms.
        """
        return self.raw["state"].get("eventduration")

    @property
    def device_mode(
        self,
    ) -> Literal[
        "dualpushbutton", "dualrocker", "singlepushbutton", "singlerocker"
    ] | None:
        """Different modes for the Hue wall switch module.

        Behavior as rocker:
          Issues a x000/x002 each time you flip the rocker (to either position).
          The event duration for the x002 is 1 (for 100ms),
          but lastupdated suggests it follows the x000 faster than that.

        Behavior as pushbutton:
          Issues a x000/x002 sequence on press/release.
          Issues a x000/x001/.../x001/x003 on press/hold/release.
          Similar to Hue remotes.

        Supported values:
        - "singlerocker"
        - "singlepushbutton"
        - "dualrocker"
        - "dualpushbutton"
        """
        return self.raw["config"].get("devicemode")

    @property
    def mode(self) -> Literal["momentary", "rocker"] | None:
        """For Ubisys S1/S2, operation mode of the switch.

        Supported values:
        - "momentary"
        - "rocker"
        """
        return self.raw["config"].get("mode")

    @property
    def window_covering_type(self) -> Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9] | None:
        """Set the covering type and starts calibration for Ubisys J1.

        Supported values:
        - 0 = Roller Shade
        - 1 = Roller Shade two motors
        - 2 = Roller Shade exterior
        - 3 = Roller Shade two motors ext
        - 4 = Drapery
        - 5 = Awning
        - 6 = Shutter
        - 7 = Tilt Blind Lift only
        - 8 = Tilt Blind lift & tilt
        - 9 = Projector Screen
        """
        return self.raw["config"].get("windowcoveringtype")

    async def set_config(
        self,
        device_mode: Literal[
            "dualpushbutton", "dualrocker", "singlepushbutton", "singlerocker"
        ]
        | None = None,
        mode: Literal["momentary", "rocker"] | None = None,
        window_covering_type: Literal[0, 1, 2, 3, 4, 5, 6, 7, 8, 9] | None = None,
    ) -> dict[str, Any]:
        """Change config of presence sensor.

        Supported values:
        - device_mode [str]
          - "dualpushbutton"
          - "dualrocker"
          - "singlepushbutton"
          - "singlerocker"
        - mode [str]
          - "momentary"
          - "rocker"
        - window_covering_type [int] 0-9
        """
        data = {
            key: value
            for key, value in {
                "devicemode": device_mode,
                "mode": mode,
                "windowcoveringtype": window_covering_type,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/config", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from collections.abc import Awaitable, Callable
from typing import Any, Final, TypedDict

from . import ResourceGroup
from .api import APIItem

RESOURCE_TYPE: Final = ResourceGroup.SCENE.value


class TypedScene(TypedDict):
    """Scene type definition."""

    id: str
    lightcount: int
    transitiontime: int
    name: str


class Scene(APIItem):
    """deCONZ scene representation.

    Dresden Elektroniks documentation of scenes in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/scenes/
    """

    raw: TypedScene

    def __init__(
        self,
        resource_id: str,
        raw: Any,
        request: Callable[..., Awaitable[dict[str, Any]]],
    ) -> None:
        """Set initial information about scene."""
        super().__init__(resource_id, raw, request)

        self.group_id: str = raw["group_id"]
        self.group_deconz_id: str = f"/groups/{self.group_id}"
        self.group_name: str = raw["group_name"]

    @property
    def resource_type(self) -> str:
        """Resource type."""
        return RESOURCE_TYPE

    async def recall(self) -> dict[str, Any]:
        """Recall scene to group."""
        return await self.request(field=f"{self.deconz_id}/recall", data={})

    async def store(self) -> dict[str, Any]:
        """Store current group state in scene.

        The actual state of each light in the group will become the lights scene state.
        """
        return await self.request(field=f"{self.deconz_id}/store", data={})

    async def set_attributes(
        self,
        name: str | None = None,
    ) -> dict[str, Any]:
        """Change attributes of scene.

        Supported values:
        - name [str]
        """
        data = {
            key: value
            for key, value in {
                "name": name,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}", data=data)

    @property
    def deconz_id(self) -> str:
        """Id to call scene over API e.g. /groups/1/scenes/1."""
        return f"{self.group_deconz_id}/{self.resource_type}/{self.id}"

    @property
    def id(self) -> str:
        """Scene ID."""
        return self.raw["id"]

    @property
    def light_count(self) -> int:
        """Lights in group."""
        return self.raw["lightcount"]

    @property
    def transition_time(self) -> int:
        """Transition time for scene."""
        return self.raw["transitiontime"]

    @property
    def name(self) -> str:
        """Scene name."""
        return self.raw["name"]

    @property
    def full_name(self) -> str:
        """Full name."""
        return f"{self.group_name} {self.name}"



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, TypedDict

from . import LightBase


class TypedCoverState(TypedDict):
    """Cover state type definition."""

    bri: int
    lift: int
    open: bool
    sat: int
    tilt: int


class TypedCover(TypedDict):
    """Cover type definition."""

    state: TypedCoverState


class Cover(LightBase):
    """Cover and Damper class.

    Position 0 means open and 100 means closed.
    """

    ZHATYPE = (
        "Level controllable output",
        "Window covering controller",
        "Window covering device",
    )

    raw: TypedCover

    @property
    def is_open(self) -> bool:
        """Is cover open."""
        if "open" not in self.raw["state"]:  # Legacy support
            return self.state is False
        return self.raw["state"]["open"]

    @property
    def lift(self) -> int:
        """Amount of closed position.

        0 is fully open.
        100 is fully closed.
        """
        if "lift" not in self.raw["state"]:  # Legacy support
            return int(self.raw["state"]["bri"] / 2.54)
        return self.raw["state"]["lift"]

    @property
    def tilt(self) -> int | None:
        """Amount of tilt.

        0 is fully open.
        100 is fully closed.
        """
        if "tilt" in self.raw["state"]:
            return self.raw["state"]["tilt"]
        elif "sat" in self.raw["state"]:  # Legacy support
            return int(self.raw["state"]["sat"] / 2.54)
        return None

    async def set_position(
        self, *, lift: int | None = None, tilt: int | None = None
    ) -> dict[str, Any]:
        """Set amount of closed position and/or tilt of cover.

        Lift [int] between 0-100.
        Scale to brightness 0-254.
        Tilt [int] between 0-100.
        Scale to saturation 0-254.
        """
        data = {}

        if lift is not None:
            if "lift" in self.raw["state"]:
                data["lift"] = lift
            elif "bri" in self.raw["state"]:  # Legacy support
                data["bri"] = int(lift * 2.54)

        if tilt is not None:
            if "tilt" in self.raw["state"]:
                data["tilt"] = tilt
            elif "sat" in self.raw["state"]:  # Legacy support
                data["sat"] = int(tilt * 2.54)

        return await self.request(field=f"{self.deconz_id}/state", data=data)

    async def open(self) -> dict[str, Any]:
        """Fully open cover."""
        data = {"open": True}
        if "open" not in self.raw["state"]:  # Legacy support
            data = {"on": False}
        return await self.request(field=f"{self.deconz_id}/state", data=data)

    async def close(self) -> dict[str, Any]:
        """Fully close cover."""
        data = {"open": False}
        if "open" not in self.raw["state"]:  # Legacy support
            data = {"on": True}
        return await self.request(field=f"{self.deconz_id}/state", data=data)

    async def stop(self) -> dict[str, Any]:
        """Stop cover motion."""
        data: dict[str, bool | int]
        data = {"stop": True}
        if "lift" not in self.raw["state"]:  # Legacy support
            data = {"bri_inc": 0}
        return await self.request(field=f"{self.deconz_id}/state", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Literal, TypedDict

from . import ALERT_KEY, ALERT_LONG, ALERT_NONE, ON_TIME_KEY, LightBase


class TypedSirenState(TypedDict):
    """Siren state type definition."""

    alert: Literal["lselect", "select", "none"]


class TypedSiren(TypedDict):
    """Siren type definition."""

    state: TypedSirenState


class Siren(LightBase):
    """Siren class."""

    ZHATYPE = ("Warning device",)

    raw: TypedSiren

    @property
    def is_on(self) -> bool:
        """If device is sounding."""
        return self.raw["state"][ALERT_KEY] == ALERT_LONG

    async def turn_on(self, duration: int | None = None) -> dict[str, Any]:
        """Turn on device.

        Duration is counted as 1/10 of a second.
        """
        data: dict[str, int | str] = {ALERT_KEY: ALERT_LONG}
        if duration:
            data[ON_TIME_KEY] = duration
        return await self.request(field=f"{self.deconz_id}/state", data=data)

    async def turn_off(self) -> dict[str, Any]:
        """Turn off device."""
        return await self.request(
            field=f"{self.deconz_id}/state",
            data={ALERT_KEY: ALERT_NONE},
        )



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import TypedDict

from . import SensorBase


class TypedFireState(TypedDict):
    """Fire state type definition."""

    fire: bool
    test: bool


class TypedFire(TypedDict):
    """Fire type definition."""

    state: TypedFireState


class Fire(SensorBase):
    """Fire sensor."""

    ZHATYPE = ("ZHAFire",)

    raw: TypedFire

    @property
    def fire(self) -> bool:
        """Fire detected."""
        return self.raw["state"]["fire"]

    @property
    def in_test_mode(self) -> bool:
        """Sensor is in test mode."""
        return self.raw["state"].get("test", False)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Literal, TypedDict

from . import LightBase


class TypedLightState(TypedDict):
    """Light state type definition."""

    alert: Literal["none", "select", "lselect"]
    bri: int
    colormode: Literal["ct", "hs", "xy"]
    ct: int
    effect: Literal["colorloop", "none"]
    hue: int
    on: bool
    sat: int
    xy: tuple[float, float]


class TypedLight(TypedDict):
    """Light type definition."""

    ctmax: int
    ctmin: int
    state: TypedLightState


class Light(LightBase):
    """deCONZ light representation.

    Dresden Elektroniks documentation of lights in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/lights/
    """

    raw: TypedLight

    @property
    def alert(self) -> Literal["none", "select", "lselect"] | None:
        """Temporary alert effect.

        Following values are possible:
        none - light is not performing an alert
        select - light is blinking a short time
        lselect - light is blinking a longer time
        """
        return self.raw["state"].get("alert")

    @property
    def brightness(self) -> int | None:
        """Brightness of the light.

        Depending on the light type 0 might not mean visible "off"
        but minimum brightness.
        """
        return self.raw["state"].get("bri")

    @property
    def color_temp(self) -> int | None:
        """Mired color temperature of the light. (2000K - 6500K)."""
        return self.raw["state"].get("ct")

    @property
    def hue(self) -> int | None:
        """Color hue of the light.

        The hue parameter in the HSV color model is between 0°-360°
        and is mapped to 0..65535 to get 16-bit resolution.
        """
        return self.raw["state"].get("hue")

    @property
    def saturation(self) -> int | None:
        """Color saturation of the light.

        There 0 means no color at all and 255 is the greatest saturation
        of the color.
        """
        return self.raw["state"].get("sat")

    @property
    def xy(self) -> tuple[float, float] | None:
        """CIE xy color space coordinates as array [x, y] of real values (0..1)."""
        x, y = self.raw["state"].get("xy", (None, None))

        if x is None or y is None:
            return None

        if x > 1:
            x = x / 65555

        if y > 1:
            y = y / 65555

        return (x, y)

    @property
    def color_mode(self) -> Literal["ct", "hs", "xy"] | None:
        """Color mode of light.

        ct - color temperature
        hs - hue and saturation
        xy - CIE xy values
        """
        return self.raw["state"].get("colormode")

    @property
    def on(self) -> bool:
        """Device state."""
        return self.raw["state"]["on"]

    @property
    def max_color_temp(self) -> int | None:
        """Max value for color temperature."""
        if (ctmax := self.raw.get("ctmax")) is not None and ctmax > 650:
            ctmax = 650
        return ctmax

    @property
    def min_color_temp(self) -> int | None:
        """Min value for color temperature."""
        if (ctmin := self.raw.get("ctmin")) is not None and ctmin < 140:
            ctmin = 140
        return ctmin

    @property
    def effect(self) -> Literal["colorloop", "none"] | None:
        """Effect of the light.

        colorloop — the light will cycle continuously through all colors
                    with the speed specified by colorloopspeed.
        none — no effect.
        """
        return self.raw["state"].get("effect")

    async def set_attributes(self, name: str) -> dict[str, Any]:
        """Change attributes of a light.

        Supported values:
        - name [str] The name of the light
        """
        data = {"name": name}
        return await self.request(field=f"{self.deconz_id}", data=data)

    async def set_state(
        self,
        alert: Literal["none", "select", "lselect"] | str | None = None,
        brightness: int | None = None,
        color_loop_speed: int | None = None,
        color_temperature: int | None = None,
        effect: Literal["colorloop", "none"] | str | None = None,
        hue: int | None = None,
        on: bool | None = None,
        on_time: int | None = None,
        saturation: int | None = None,
        transition_time: int | None = None,
        xy: tuple[float, float] | None = None,
    ) -> dict[str, Any]:
        """Change state of a light.

        Supported values:
        - alert [str]
          - "none" light is not performing an alert
          - "select" light is blinking a short time
          - "lselect" light is blinking a longer time
        - brightness [int] 0-255
        - color_loop_speed [int] 1-255
          - 1 = very fast
          - 15 is default
          - 255 very slow
        - color_temperature [int] between ctmin-ctmax
        - effect [str]
          - "none" no effect
          - "colorloop" the light will cycle continuously through all
                        colors with the speed specified by colorloopspeed
        - hue [int] 0-65535
        - on [bool] True/False
        - on_time [int] 0-65535 1/10 seconds resolution
        - saturation [int] 0-255
        - transition_time [int] 0-65535 1/10 seconds resolution
        - xy [tuple] 0-1
        """
        data = {
            key: value
            for key, value in {
                "alert": alert,
                "bri": brightness,
                "colorloopspeed": color_loop_speed,
                "ct": color_temperature,
                "effect": effect,
                "hue": hue,
                "on": on,
                "ontime": on_time,
                "sat": saturation,
                "transitiontime": transition_time,
                "xy": xy,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/state", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import Any, Literal, TypedDict

from . import SensorBase


class TypedDoorLockConfig(TypedDict):
    """Door lock config type definition."""

    lock: bool


class TypedDoorLockState(TypedDict):
    """Door lock state type definition."""

    lockstate: Literal["locked", "unlocked", "undefined", "not fully locked"]


class TypedDoorLock(TypedDict):
    """Door lock type definition."""

    config: TypedDoorLockConfig
    state: TypedDoorLockState


class DoorLock(SensorBase):
    """Door lock sensor."""

    ZHATYPE = ("ZHADoorLock",)

    raw: TypedDoorLock

    @property
    def is_locked(self) -> bool:
        """Return True if lock is locked."""
        return self.lock_state == "locked"

    @property
    def lock_state(
        self,
    ) -> Literal["locked", "unlocked", "undefined", "not fully locked"]:
        """State the lock is in.

        Supported values:
        - "locked"
        - "unlocked"
        - "undefined"
        - "not fully locked"
        """
        return self.raw["state"]["lockstate"]

    @property
    def lock_configuration(self) -> bool:
        """Lock configuration."""
        return self.raw["config"]["lock"]

    async def lock(self) -> dict[str, Any]:
        """Lock the lock."""
        return await self.request(
            field=f"{self.deconz_id}/config",
            data={"lock": True},
        )

    async def unlock(self) -> dict[str, Any]:
        """Unlock the lock."""
        return await self.request(
            field=f"{self.deconz_id}/config",
            data={"lock": False},
        )



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Final, Literal, TypedDict

from . import ResourceGroup
from .api import APIItem

RESOURCE_TYPE: Final = ResourceGroup.ALARM.value

PATH_ARM_AWAY: Final = "arm_away"
PATH_ARM_NIGHT: Final = "arm_night"
PATH_ARM_STAY: Final = "arm_stay"
PATH_DISARM: Final = "disarm"

ARM_MODE_ARMED_AWAY: Final = "armed_away"
ARM_MODE_ARMED_NIGHT: Final = "armed_night"
ARM_MODE_ARMED_STAY: Final = "armed_stay"
ARM_MODE_DISARMED: Final = "disarmed"

ARM_STATE_ARMED_AWAY: Final = "armed_away"
ARM_STATE_ARMED_NIGHT: Final = "armed_night"
ARM_STATE_ARMED_STAY: Final = "armed_stay"
ARM_STATE_ARMING_AWAY: Final = "arming_away"
ARM_STATE_ARMING_NIGHT: Final = "arming_night"
ARM_STATE_ARMING_STAY: Final = "arming_stay"
ARM_STATE_DISARMED: Final = "disarmed"
ARM_STATE_ENTRY_DELAY: Final = "entry_delay"
ARM_STATE_EXIT_DELAY: Final = "exit_delay"
ARM_STATE_IN_ALARM: Final = "in_alarm"

DEVICE_TRIGGER_ACTION: Final = "state/action"
DEVICE_TRIGGER_BUTTON_EVENT: Final = "state/buttonevent"
DEVICE_TRIGGER_ON: Final = "state/on"
DEVICE_TRIGGER_OPEN: Final = "state/open"
DEVICE_TRIGGER_PRESENCE: Final = "state/presence"
DEVICE_TRIGGER_VIBRATION: Final = "state/vibration"


class TypedAlarmSystemConfig(TypedDict):
    """Alarm system config type definition."""

    armmode: Literal["armed_away", "armed_night", "armed_stay", "disarmed"]
    configured: bool
    disarmed_entry_delay: int
    disarmed_exit_delay: int
    armed_away_entry_delay: int
    armed_away_exit_delay: int
    armed_away_trigger_duration: int
    armed_stay_entry_delay: int
    armed_stay_exit_delay: int
    armed_stay_trigger_duration: int
    armed_night_entry_delay: int
    armed_night_exit_delay: int
    armed_night_trigger_duration: int


class TypedAlarmSystemState(TypedDict):
    """Alarm system state type definition."""

    armstate: Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "arming_away",
        "arming_night",
        "arming_stay",
        "disarmed",
        "entry_delay",
        "exit_delay",
        "in_alarm",
    ]
    seconds_remaining: int


class TypedAlarmSystemDevices(TypedDict):
    """Alarm system device type definition."""

    armmask: str
    trigger: str


class TypedAlarmSystem(TypedDict):
    """Alarm system type definition."""

    name: str
    config: TypedAlarmSystemConfig
    state: TypedAlarmSystemState
    devices: dict[str, TypedAlarmSystemDevices]


class AlarmSystem(APIItem):
    """deCONZ alarm system representation.

    Dresden Elektroniks documentation of alarm systems in deCONZ
    https://dresden-elektronik.github.io/deconz-rest-doc/endpoints/alarmsystems/
    """

    raw: TypedAlarmSystem

    @property
    def resource_type(self) -> str:
        """Resource type."""
        return RESOURCE_TYPE

    @property
    def deconz_id(self) -> str:
        """Id to call alarm system over API e.g. /alarmsystems/1."""
        return f"/{self.resource_type}/{self.resource_id}"

    async def set_alarm_system_configuration(
        self,
        code0: str | None = None,
        armed_away_entry_delay: int | None = None,
        armed_away_exit_delay: int | None = None,
        armed_away_trigger_duration: int | None = None,
        armed_night_entry_delay: int | None = None,
        armed_night_exit_delay: int | None = None,
        armed_night_trigger_duration: int | None = None,
        armed_stay_entry_delay: int | None = None,
        armed_stay_exit_delay: int | None = None,
        armed_stay_trigger_duration: int | None = None,
        disarmed_entry_delay: int | None = None,
        disarmed_exit_delay: int | None = None,
    ) -> dict[str, Any]:
        """Set config of alarm system."""
        data = {
            key: value
            for key, value in {
                "code0": code0,
                "armed_away_entry_delay": armed_away_entry_delay,
                "armed_away_exit_delay": armed_away_exit_delay,
                "armed_away_trigger_duration": armed_away_trigger_duration,
                "armed_night_entry_delay": armed_night_entry_delay,
                "armed_night_exit_delay": armed_night_exit_delay,
                "armed_night_trigger_duration": armed_night_trigger_duration,
                "armed_stay_entry_delay": armed_stay_entry_delay,
                "armed_stay_exit_delay": armed_stay_exit_delay,
                "armed_stay_trigger_duration": armed_stay_trigger_duration,
                "disarmed_entry_delay": disarmed_entry_delay,
                "disarmed_exit_delay": disarmed_exit_delay,
            }.items()
            if value is not None
        }
        return await self._request(
            "put",
            path=f"{self.deconz_id}/config",
            json=data,
        )

    async def arm_away(self, pin_code: str) -> dict[str, Any]:
        """Set the alarm to away."""
        return await self._request(
            "put",
            path=f"{self.deconz_id}/{PATH_ARM_AWAY}",
            json={"code0": pin_code},
        )

    async def arm_night(self, pin_code: str) -> dict[str, Any]:
        """Set the alarm to night."""
        return await self._request(
            "put",
            path=f"{self.deconz_id}/{PATH_ARM_NIGHT}",
            json={"code0": pin_code},
        )

    async def arm_stay(self, pin_code: str) -> dict[str, Any]:
        """Set the alarm to stay."""
        return await self._request(
            "put",
            path=f"{self.deconz_id}/{PATH_ARM_STAY}",
            json={"code0": pin_code},
        )

    async def disarm(self, pin_code: str) -> dict[str, Any]:
        """Disarm alarm."""
        return await self._request(
            "put",
            path=f"{self.deconz_id}/{PATH_DISARM}",
            json={"code0": pin_code},
        )

    @property
    def arm_state(
        self,
    ) -> Literal[
        "armed_away",
        "armed_night",
        "armed_stay",
        "arming_away",
        "arming_night",
        "arming_stay",
        "disarmed",
        "entry_delay",
        "exit_delay",
        "in_alarm",
    ]:
        """Alarm system state.

        Can be different from the config.armmode during state transitions.

        Supported values:
        - "armed_away"
        - "armed_night"
        - "armed_stay"
        - "arming_away"
        - "arming_night"
        - "arming_stay"
        - "disarmed"
        - "entry_delay"
        - "exit_delay"
        - "in_alarm"
        """
        return self.raw["state"]["armstate"]

    @property
    def seconds_remaining(self) -> int:
        """Remaining time while armstate in "exit_delay" or "entry_delay" state.

        In all other states the value is 0.

        Supported values:
          0-255.
        """
        return self.raw["state"]["seconds_remaining"]

    @property
    def pin_configured(self) -> bool:
        """Is PIN code configured."""
        return self.raw["config"]["configured"]

    @property
    def arm_mode(
        self,
    ) -> Literal["armed_away", "armed_night", "armed_stay", "disarmed"]:
        """Target arm mode.

        Supported values:
        - "armed_away"
        - "armed_night"
        - "armed_stay"
        - "disarmed"
        """
        return self.raw["config"]["armmode"]

    @property
    def armed_away_entry_delay(self) -> int:
        """Delay in seconds before an alarm is triggered.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_away_entry_delay"]

    @property
    def armed_away_exit_delay(self) -> int:
        """Delay in seconds before an alarm is armed.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_away_exit_delay"]

    @property
    def armed_away_trigger_duration(self) -> int:
        """Duration of alarm trigger.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_away_trigger_duration"]

    @property
    def armed_night_entry_delay(self) -> int:
        """Delay in seconds before an alarm is triggered.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_night_entry_delay"]

    @property
    def armed_night_exit_delay(self) -> int:
        """Delay in seconds before an alarm is armed.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_night_exit_delay"]

    @property
    def armed_night_trigger_duration(self) -> int:
        """Duration of alarm trigger.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_night_trigger_duration"]

    @property
    def armed_stay_entry_delay(self) -> int:
        """Delay in seconds before an alarm is triggered.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_stay_entry_delay"]

    @property
    def armed_stay_exit_delay(self) -> int:
        """Delay in seconds before an alarm is armed.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_stay_exit_delay"]

    @property
    def armed_stay_trigger_duration(self) -> int:
        """Duration of alarm trigger.

        Supported values:
          0-255.
        """
        return self.raw["config"]["armed_stay_trigger_duration"]

    @property
    def disarmed_entry_delay(self) -> int:
        """Delay in seconds before an alarm is triggered.

        Supported values:
          0-255.
        """
        return self.raw["config"]["disarmed_entry_delay"]

    @property
    def disarmed_exit_delay(self) -> int:
        """Delay in seconds before an alarm is armed.

        Supported values:
          0-255.
        """
        return self.raw["config"]["disarmed_exit_delay"]

    @property
    def devices(self) -> dict[str, Any]:
        """Devices associated with the alarm system.

        The keys refer to the uniqueid of a light, sensor, or keypad.
        Dictionary values:
        - armmask - A combination of arm modes in which the device triggers alarms.
          A — armed_away
          N — armed_night
          S — armed_stay
          "none" — for keypads and keyfobs
        - trigger - Specifies arm modes in which the device triggers alarms.
          "state/presence"
          "state/open"
          "state/vibration"
          "state/buttonevent"
          "state/on"
        """
        return self.raw["devices"]

    async def add_device(
        self,
        unique_id: str,
        armed_away: bool = False,
        armed_night: bool = False,
        armed_stay: bool = False,
        trigger: Literal[
            "state/presence",
            "state/open",
            "state/vibration",
            "state/buttonevent",
            "state/on",
        ]
        | None = None,
        is_keypad: bool = False,
    ) -> dict[str, Any]:
        """Link device with alarm system.

        A device can be linked to exactly one alarm system.
          If it is added to another alarm system, it is automatically removed
          from the prior one.
        This request is used for adding and also for updating a device entry.
        The uniqueid refers to sensors, lights or keypads.
          Adding a light can be useful, e.g. when an alarm should be triggered,
          after a light is powered or switched on in the basement.
        """
        data = {"armmask": ""}
        data["armmask"] += "A" if armed_away else ""
        data["armmask"] += "N" if armed_night else ""
        data["armmask"] += "S" if armed_stay else ""

        if trigger:
            data["trigger"] = trigger

        if is_keypad:
            data = {}

        return await self._request(
            "put",
            path=f"{self.deconz_id}/device/{unique_id}",
            json=data,
        )

    async def remove_device(self, unique_id: str) -> dict[str, Any]:
        """Unlink device with alarm system."""
        return await self._request(
            "delete",
            path=f"{self.deconz_id}/device/{unique_id}",
        )



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from collections.abc import Callable
import logging
from pprint import pformat
from typing import Any, Literal

import aiohttp

from .config import Config
from .errors import RequestError, ResponseError, raise_error
from .interfaces.alarm_systems import AlarmSystems
from .interfaces.events import EventHandler
from .interfaces.groups import Groups
from .interfaces.lights import LightResourceManager
from .interfaces.scenes import Scenes
from .interfaces.sensors import SensorResourceManager
from .models import ResourceGroup
from .models.event import EventType
from .models.light.light import Light
from .websocket import SIGNAL_CONNECTION_STATE, SIGNAL_DATA, STATE_RUNNING, WSClient

LOGGER = logging.getLogger(__name__)


class DeconzSession:
    """deCONZ representation that handles lights, groups, scenes and sensors."""

    def __init__(
        self,
        session: aiohttp.ClientSession,
        host: str,
        port: int,
        api_key: str | None = None,
        add_device: Callable[[str, Any], None] | None = None,
        connection_status: Callable[[bool], None] | None = None,
        legacy_add_device: bool = True,
        legacy_update_group_color: bool = True,
    ) -> None:
        """Session setup."""
        self.session = session
        self.host = host
        self.port = port
        self.api_key = api_key

        self.add_device_callback = add_device
        self.connection_status_callback = connection_status

        self.config = Config({}, self.request)
        self.events = EventHandler(self)
        self.websocket: WSClient | None = None

        self.alarmsystems = AlarmSystems(self)
        self.groups = Groups(self)
        self.lights = LightResourceManager(self)
        self.scenes = Scenes(self)
        self.sensors = SensorResourceManager(self)

        self.alarmsystems.post_init()
        self.groups.post_init()
        self.lights.post_init()
        self.scenes.post_init()
        self.sensors.post_init()

        if legacy_add_device:
            self.legacy_add_device_callback()
        if legacy_update_group_color:
            self.legacy_update_group_color()

    def legacy_add_device_callback(self) -> None:
        """Support legacy way to signal new device."""

        def signal_new_device(resource: ResourceGroup, device: Any) -> None:
            """Emit signal new device has been added."""
            if self.add_device_callback:
                self.add_device_callback(resource.value, device)

        def signal_new_alarm(event_type: EventType, alarm_id: str) -> None:
            """Signal new alarm system."""
            signal_new_device(ResourceGroup.ALARM, self.alarmsystems[alarm_id])

        def signal_new_group(event_type: EventType, group_id: str) -> None:
            """Signal new group."""
            signal_new_device(ResourceGroup.GROUP, self.groups[group_id])

        def signal_new_light(event_type: EventType, light_id: str) -> None:
            """Signal new light."""
            signal_new_device(ResourceGroup.LIGHT, self.lights[light_id])

        def signal_new_sensor(event_type: EventType, sensor_id: str) -> None:
            """Signal new sensor."""
            signal_new_device(ResourceGroup.SENSOR, self.sensors[sensor_id])

        self.alarmsystems.subscribe(signal_new_alarm, EventType.ADDED)
        self.groups.subscribe(signal_new_group, EventType.ADDED)
        self.lights.subscribe(signal_new_light, EventType.ADDED)
        self.sensors.subscribe(signal_new_sensor, EventType.ADDED)

    def legacy_update_group_color(self) -> None:
        """Support legacy way to update group colors."""

        lights = self.lights.lights

        def signal_new_light(event_type: EventType, light_id: str) -> None:
            """Signal new light."""

            light = lights[light_id]

            def updated_light() -> None:
                """Emit signal new device has been added."""
                if "attr" not in light.changed_keys:
                    self.update_group_color([light_id])

            light.subscribe(updated_light)

        lights.subscribe(signal_new_light, EventType.ADDED)

    async def get_api_key(
        self,
        api_key: str | None = None,
        client_name: str = "pydeconz",
    ) -> str:
        """Request a new API key.

        Supported values:
        - api_key [str] 10-40 characters, key to use for authentication
        - client_name [str] 0-40 characters, name of the client application
        """
        data = {
            key: value
            for key, value in {
                "username": api_key,
                "devicetype": client_name,
            }.items()
            if value is not None
        }
        response: list[dict[str, dict[str, str]]] = await self._request(
            "post",
            url=f"http://{self.host}:{self.port}/api",
            json=data,
        )

        return response[0]["success"]["username"]

    def start(self, websocketport: int | None = None) -> None:
        """Connect websocket to deCONZ."""
        if self.config.websocket_port is not None:
            websocketport = self.config.websocket_port

        if not websocketport:
            LOGGER.error("No websocket port specified")
            return

        self.websocket = WSClient(
            self.session, self.host, websocketport, self.session_handler
        )
        self.websocket.start()

    def close(self) -> None:
        """Close websession and websocket to deCONZ."""
        if self.websocket:
            self.websocket.stop()

    async def refresh_state(self) -> None:
        """Read deCONZ parameters."""
        data = await self.request("get", "")

        self.config.raw.update(data[ResourceGroup.CONFIG.value])

        self.alarmsystems.process_raw(data.get(ResourceGroup.ALARM.value, {}))
        self.groups.process_raw(data[ResourceGroup.GROUP.value])
        self.lights.process_raw(data[ResourceGroup.LIGHT.value])
        self.sensors.process_raw(data[ResourceGroup.SENSOR.value])

        self.update_group_color(list(self.lights.keys()))

    async def request(
        self,
        method: str,
        path: str,
        json: dict[str, Any] | None = None,
    ) -> dict[str, Any]:
        """Make a request to the API."""
        response: dict[str, Any] = await self._request(
            method,
            url=f"http://{self.host}:{self.port}/api/{self.api_key}{path}",
            json=json,
        )
        return response

    async def _request(
        self,
        method: str,
        url: str,
        json: dict[str, Any] | None = None,
    ) -> Any:
        """Make a request."""
        LOGGER.debug('Sending "%s" "%s" to "%s"', method, json, url)

        try:
            async with self.session.request(method, url, json=json) as res:

                if res.content_type != "application/json":
                    raise ResponseError(
                        "Invalid content type: {}".format(res.content_type)
                    )

                response = await res.json()
                LOGGER.debug("HTTP request response: %s", pformat(response))

                _raise_on_error(response)

                return response

        except aiohttp.client_exceptions.ClientError as err:
            raise RequestError(
                "Error requesting data from {}: {}".format(self.host, err)
            ) from None

    async def session_handler(self, signal: Literal["data", "state"]) -> None:
        """Signalling from websocket.

        data - new data available for processing.
        state - network state has changed.
        """
        if not self.websocket:
            return

        if signal == SIGNAL_DATA:
            self.events.handler(self.websocket.data)

        elif signal == SIGNAL_CONNECTION_STATE and self.connection_status_callback:
            self.connection_status_callback(self.websocket.state == STATE_RUNNING)

    def update_group_color(self, lights: list[str]) -> None:
        """Update group colors based on light states.

        deCONZ group updates don't contain any information about the current
        state of the lights in the group. This method updates the color
        properties of the group to the current color of the lights in the
        group.
        """
        for group in self.groups.values():
            # Skip group if there are no common light ids.
            if not any({*lights} & {*group.lights}):
                continue

            # More than one light means self.initialize called this method.
            if len(light_ids := lights) > 1:
                light_ids = group.lights

            first = True
            for light_id in light_ids:
                light = self.lights.lights[light_id]

                if light.ZHATYPE == Light.ZHATYPE and light.reachable:
                    group.update_color_state(light, update_all_attributes=first)
                    first = False


def _raise_on_error(data: list[dict[str, Any]] | dict[str, Any]) -> None:
    """Check response for error message."""
    if isinstance(data, list) and data:
        data = data[0]

    if isinstance(data, dict) and "error" in data:
        raise_error(data["error"])



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, Final, TypedDict

from . import SensorBase

PRESENCE_DELAY: Final = "delay"
PRESENCE_DURATION: Final = "duration"
PRESENCE_SENSITIVITY: Final = "sensitivity"
PRESENCE_SENSITIVITY_MAX: Final = "sensitivitymax"
PRESENCE_DARK: Final = "dark"
PRESENCE_PRESENCE: Final = "presence"


class TypedPresenceConfig(TypedDict):
    """Presence config type definition."""

    delay: int
    duration: int
    sensitivity: int
    sensitivitymax: int


class TypedPresenceState(TypedDict):
    """Presence state type definition."""

    dark: bool
    presence: bool


class TypedPresence(TypedDict):
    """Presence type definition."""

    config: TypedPresenceConfig
    state: TypedPresenceState


class Presence(SensorBase):
    """Presence detector."""

    ZHATYPE = ("ZHAPresence", "CLIPPresence")

    raw: TypedPresence

    @property
    def dark(self) -> bool | None:
        """If the area near the sensor is light or not."""
        return self.raw["state"].get(PRESENCE_DARK)

    @property
    def delay(self) -> int | None:
        """Occupied to unoccupied delay in seconds."""
        return self.raw["config"].get(PRESENCE_DELAY)

    @property
    def duration(self) -> int | None:
        """Minimum duration which presence will be true."""
        return self.raw["config"].get(PRESENCE_DURATION)

    @property
    def presence(self) -> bool:
        """Motion detected."""
        return self.raw["state"][PRESENCE_PRESENCE]

    @property
    def sensitivity(self) -> int | None:
        """Sensitivity setting for Philips Hue motion sensor.

        Supported values:
        - 0-[sensitivitymax]
        """
        return self.raw["config"].get(PRESENCE_SENSITIVITY)

    @property
    def max_sensitivity(self) -> int | None:
        """Maximum sensitivity value."""
        return self.raw["config"].get(PRESENCE_SENSITIVITY_MAX)

    async def set_config(
        self,
        delay: int | None = None,
        duration: int | None = None,
        sensitivity: int | None = None,
    ) -> dict[str, Any]:
        """Change config of presence sensor.

        Supported values:
        - delay [int] 0-65535 (in seconds)
        - duration [int] 0-65535 (in seconds)
        - sensitivity [int] 0-[sensitivitymax]
        """
        data = {
            key: value
            for key, value in {
                PRESENCE_DELAY: delay,
                PRESENCE_DURATION: duration,
                PRESENCE_SENSITIVITY: sensitivity,
            }.items()
            if value is not None
        }
        return await self.request(field=f"{self.deconz_id}/config", data=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import TypedDict

from . import SensorBase


class TypedConsumptionState(TypedDict):
    """Consumption state type definition."""

    consumption: int
    power: int


class TypedConsumption(TypedDict):
    """Consumption type definition."""

    state: TypedConsumptionState


class Consumption(SensorBase):
    """Power consumption sensor."""

    ZHATYPE = ("ZHAConsumption",)

    raw: TypedConsumption

    @property
    def consumption(self) -> int:
        """Consumption."""
        return self.raw["state"]["consumption"]

    @property
    def scaled_consumption(self) -> float:
        """State of sensor."""
        return self.consumption / 1000

    @property
    def power(self) -> int | None:
        """Power."""
        return self.raw["state"].get("power")



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Any, TypedDict

from . import LightBase


class TypedLockState(TypedDict):
    """Lock state type definition."""

    on: bool


class TypedLock(TypedDict):
    """Lock type definition."""

    state: TypedLockState


class Lock(LightBase):
    """Lock class."""

    ZHATYPE = ("Door Lock",)

    raw: TypedLock

    @property
    def is_locked(self) -> bool:
        """State of lock."""
        return self.raw["state"]["on"]

    async def lock(self) -> dict[str, Any]:
        """Lock the lock."""
        return await self.request(field=f"{self.deconz_id}/state", data={"on": True})

    async def unlock(self) -> dict[str, Any]:
        """Unlock the lock."""
        return await self.request(field=f"{self.deconz_id}/state", data={"on": False})



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import Final, TypedDict

from . import SensorBase

DAYLIGHT_STATUS: Final = {
    100: "nadir",
    110: "night_end",
    120: "nautical_dawn",
    130: "dawn",
    140: "sunrise_start",
    150: "sunrise_end",
    160: "golden_hour_1",
    170: "solar_noon",
    180: "golden_hour_2",
    190: "sunset_start",
    200: "sunset_end",
    210: "dusk",
    220: "nautical_dusk",
    230: "night_start",
}


class TypedDaylightConfig(TypedDict):
    """Daylight config type definition."""

    configured: bool
    sunriseoffset: int
    sunsetoffset: int


class TypedDaylightState(TypedDict):
    """Daylight state type definition."""

    daylight: bool
    status: int


class TypedDaylight(TypedDict):
    """Daylight type definition."""

    config: TypedDaylightConfig
    state: TypedDaylightState


class Daylight(SensorBase):
    """Daylight sensor built into deCONZ software."""

    ZHATYPE = ("Daylight",)

    raw: TypedDaylight

    @property
    def configured(self) -> bool:
        """Is daylight sensor configured."""
        return self.raw["config"]["configured"]

    @property
    def daylight(self) -> bool:
        """Is daylight."""
        return self.raw["state"]["daylight"]

    @property
    def status(self) -> str:
        """Return the daylight status string."""
        return DAYLIGHT_STATUS.get(self.raw["state"]["status"], "unknown")

    @property
    def sunrise_offset(self) -> int:
        """Sunrise offset.

        -120 to 120.
        """
        return self.raw["config"]["sunriseoffset"]

    @property
    def sunset_offset(self) -> int:
        """Sunset offset.

        -120 to 120.
        """
        return self.raw["config"]["sunsetoffset"]



"""Setup for pydeCONZ."""

from setuptools import find_packages, setup

MIN_PY_VERSION = "3.9"
PACKAGES = find_packages(exclude=["tests", "tests.*"])
REQUIREMENTS = list(val.strip() for val in open("requirements.txt"))
VERSION = "88"

setup(
    name="pydeconz",
    packages=PACKAGES,
    version=VERSION,
    description="A Python library for communicating with deCONZ REST-API from Dresden Elektronik",
    author="Robert Svensson",
    author_email="Kane610@users.noreply.github.com",
    license="MIT",
    url="https://github.com/Kane610/deconz",
    download_url=f"https://github.com/Kane610/deconz/archive/v{VERSION}.tar.gz",
    install_requires=REQUIREMENTS,
    tests_require=["pytest-aiohttp", "pytest", "aioresponses"],
    keywords=["deconz", "zigbee", "homeassistant"],
    classifiers=["Natural Language :: English", "Programming Language :: Python :: 3"],
    python_requires=f">={MIN_PY_VERSION}",
)



"""API base classes."""

from __future__ import annotations

from asyncio import CancelledError, Task, create_task, sleep
from collections.abc import Awaitable, Callable
import logging
from typing import Any

from ..errors import BridgeBusy

LOGGER = logging.getLogger(__name__)

SubscriptionType = Callable[..., None]


class APIItem:
    """Base class for an API item."""

    def __init__(
        self,
        resource_id: str,
        raw: Any,
        request: Callable[..., Awaitable[dict[str, Any]]],
    ) -> None:
        """Initialize API item."""
        self.resource_id = resource_id
        self.raw = raw
        self._request = request

        self.changed_keys: set[str] = set()

        self._callbacks: list[SubscriptionType] = []
        self._subscribers: list[SubscriptionType] = []
        self._sleep_task: Task[Callable[..., Any]] | None = None

        self.post_init()

    def post_init(self) -> None:
        """Post init method used by subclasses."""

    def register_callback(self, callback: SubscriptionType) -> None:
        """Register callback for signalling."""
        self._callbacks.append(callback)

    def remove_callback(self, callback: SubscriptionType) -> None:
        """Remove callback previously registered."""
        if callback in self._callbacks:
            self._callbacks.remove(callback)

    def subscribe(self, callback: SubscriptionType) -> Callable[[], None]:
        """Subscribe to events.

        Return function to unsubscribe.
        """
        self._subscribers.append(callback)

        def unsubscribe() -> None:
            """Unsubscribe callback."""
            self._subscribers.remove(callback)

        return unsubscribe

    def update(self, raw: dict[str, dict[str, Any]]) -> None:
        """Update input attr in self.

        Store a set of keys with changed values.
        """
        changed_keys = set()

        for k, v in raw.items():
            changed_keys.add(k)

            if isinstance(self.raw.get(k), dict) and isinstance(v, dict):
                changed_keys.update(set(v.keys()))
                self.raw[k].update(v)

            else:
                self.raw[k] = v

        self.changed_keys = changed_keys

        for callback in self._callbacks + self._subscribers:
            callback()

    async def request(
        self, field: str, data: dict[str, Any], tries: int = 0
    ) -> dict[str, Any]:
        """Set state of device."""
        if self._sleep_task is not None:
            self._sleep_task.cancel()
            self._sleep_task = None

        try:
            return await self._request("put", path=field, json=data)

        except BridgeBusy:
            LOGGER.debug("Bridge is busy, schedule retry %s %s", field, str(data))

            if (tries := tries + 1) < 3:
                self._sleep_task = create_task(sleep(2 ** (tries)))

                try:
                    await self._sleep_task
                except CancelledError:
                    return {}

                return await self.request(field, data, tries)

            self._sleep_task = None
            raise BridgeBusy



"""Python library to connect deCONZ and Home Assistant to work together."""

from .api import APIItem


class DeconzDevice(APIItem):
    """deCONZ resource base representation.

    Dresden Elektroniks REST API documentation
    http://dresden-elektronik.github.io/deconz-rest-doc/
    """

    @property
    def resource_type(self) -> str:
        """Resource type, e.g. alarmsystems/groups/lights/scenes/sensors."""

    @property
    def deconz_id(self) -> str:
        """Id to call device over API e.g. /sensors/1."""
        return f"/{self.resource_type}/{self.resource_id}"

    @property
    def etag(self) -> str:
        """HTTP etag change on any action to the device."""
        raw: dict[str, str] = self.raw
        return raw.get("etag", "")

    @property
    def manufacturer(self) -> str:
        """Device manufacturer."""
        raw: dict[str, str] = self.raw
        return raw.get("manufacturername", "")

    @property
    def model_id(self) -> str:
        """Device model."""
        raw: dict[str, str] = self.raw
        return raw.get("modelid", "")

    @property
    def name(self) -> str:
        """Name of the device."""
        raw: dict[str, str] = self.raw
        return raw.get("name", "")

    @property
    def software_version(self) -> str:
        """Firmware version."""
        raw: dict[str, str] = self.raw
        return raw.get("swversion", "")

    @property
    def type(self) -> str:
        """Human readable type of the device."""
        raw: dict[str, str] = self.raw
        return raw.get("type", "")

    @property
    def unique_id(self) -> str:
        """Id for unique device identification."""
        raw: dict[str, str] = self.raw
        return raw.get("uniqueid", "")



"""Python library to connect deCONZ and Home Assistant to work together."""

# Legacy imports kept for backwards compatibility

from .interfaces.groups import *  # noqa: F401, F403
from .models.group import *  # noqa: F401, F403



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import Final

from .. import ResourceGroup
from ..deconz_device import DeconzDevice

RESOURCE_TYPE: Final = ResourceGroup.LIGHT.value

ALERT_KEY: Final = "alert"
ALERT_LONG: Final = "lselect"
ALERT_NONE: Final = "none"
ALERT_SHORT: Final = "select"

EFFECT_NONE: Final = "none"
EFFECT_COLOR_LOOP: Final = "colorloop"

ON_TIME_KEY: Final = "ontime"


class LightBase(DeconzDevice):
    """deCONZ light representation.

    Dresden Elektroniks documentation of lights in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/lights/
    """

    ZHATYPE: tuple[str, ...] = ()

    @property
    def resource_type(self) -> str:
        """Resource type."""
        return RESOURCE_TYPE

    @property
    def state(self) -> bool | None:
        """Device state."""
        raw: dict[str, bool] = self.raw["state"]
        return raw.get("on")

    @property
    def reachable(self) -> bool:
        """Is light reachable."""
        raw: dict[str, bool] = self.raw["state"]
        return raw["reachable"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from collections.abc import Awaitable, Callable
from typing import Any, Final, Literal

from .utils import normalize_bridge_id

UNINITIALIZED_BRIDGE_ID: Final = "0000000000000000"


class Config:
    """deCONZ configuration representation.

    Dresden Elektroniks documentation of config in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/config/
    """

    def __init__(
        self,
        raw: dict[str, Any],
        request: Callable[..., Awaitable[dict[str, Any]]],
    ) -> None:
        """Set configuration about deCONZ gateway."""
        self.raw = raw
        self.request = request

    @property
    def api_version(self) -> str | None:
        """Version of the deCONZ Rest API."""
        return self.raw.get("apiversion")

    @property
    def bridge_id(self) -> str:
        """Gateway unique identifier."""
        return normalize_bridge_id(self.raw.get("bridgeid", UNINITIALIZED_BRIDGE_ID))

    @property
    def device_name(self) -> str | None:
        """Product name of the gateway.

        Valid values are "ConBee", "RaspBee", "ConBee II" and "RaspBee II".
        """
        return self.raw.get("devicename")

    @property
    def dhcp(self) -> bool | None:
        """Whether the IP address of the bridge is obtained with DHCP."""
        return self.raw.get("dhcp")

    @property
    def firmware_version(self) -> str | None:
        """Version of the ZigBee firmware."""
        return self.raw.get("fwversion")

    @property
    def gateway(self) -> str | None:
        """IPv4 address of the gateway."""
        return self.raw.get("gateway")

    @property
    def ip_address(self) -> str | None:
        """IPv4 address of the gateway."""
        return self.raw.get("ipaddress")

    @property
    def link_button(self) -> bool | None:
        """Is gateway unlocked."""
        return self.raw.get("linkbutton")

    @property
    def local_time(self) -> bool | None:
        """Localtime of the gateway."""
        return self.raw.get("localtime")

    @property
    def mac(self) -> str | None:
        """MAC address of gateway."""
        return self.raw.get("mac")

    @property
    def model_id(self) -> str | None:
        """Model describing either conbee or raspbee."""
        return self.raw.get("modelid")

    @property
    def name(self) -> str | None:
        """Name of the gateway."""
        return self.raw.get("name")

    @property
    def network_mask(self) -> str | None:
        """Network mask of the gateway."""
        return self.raw.get("netmask")

    @property
    def network_open_duration(self) -> int | None:
        """Duration in seconds used by lights and sensors search."""
        return self.raw.get("networkopenduration")

    @property
    def ntp(self) -> Literal["synced", "unsynced"] | None:
        """Tells if the NTP time is "synced" or "unsynced".

        Only for gateways running on Linux.
        """
        return self.raw.get("ntp")

    @property
    def pan_id(self) -> int | None:
        """Zigbee pan ID of the gateway."""
        return self.raw.get("panid")

    @property
    def portal_services(self) -> bool | None:
        """State of registration to portal service.

        Is the bridge registered to synchronize data with a portal account.
        """
        return self.raw.get("portalservices")

    @property
    def rf_connected(self) -> bool | None:
        """State of deCONZ connection to firmware and if Zigbee network is up."""
        return self.raw.get("rfconnected")

    @property
    def software_update(self) -> dict[str, Any] | None:
        """Contains information related to software updates."""
        return self.raw.get("swupdate")

    @property
    def software_version(self) -> str | None:
        """Software version of the gateway."""
        return self.raw.get("swversion")

    @property
    def time_format(self) -> Literal["12h", "24h"] | None:
        """Timeformat used by gateway.

        Supported values:
        "12h" or "24h"
        """
        return self.raw.get("timeformat")

    @property
    def time_zone(self) -> str | None:
        """Time zone used by gateway.

        Only on Raspberry Pi.
        "None" if not further specified.
        """
        return self.raw.get("timezone")

    @property
    def utc(self) -> str | None:
        """UTC time of gateway in ISO 8601 format."""
        return self.raw.get("utc")

    @property
    def uuid(self) -> str | None:
        """UPNP Unique ID of the gateway."""
        return self.raw.get("uuid")

    @property
    def websocket_notify_all(self) -> bool | None:
        """All state changes will be signalled through the Websocket connection.

        Default true.
        """
        return self.raw.get("websocketnotifyall")

    @property
    def websocket_port(self) -> int | None:
        """Websocket port."""
        return self.raw.get("websocketport")

    @property
    def whitelist(self) -> dict[str, Any] | None:
        """Array of whitelisted API keys."""
        return self.raw.get("whitelist")

    @property
    def zigbee_channel(self) -> Literal[11, 15, 20, 25] | None:
        """Wireless frequency channel.

        Supported channels: 11, 15, 20, 25.
        """
        return self.raw.get("zigbeechannel")

    async def set_config(
        self,
        discovery: bool | None = None,
        group_delay: int | None = None,
        light_last_seen_interval: int | None = None,
        name: str | None = None,
        network_open_duration: int | None = None,
        otau_active: bool | None = None,
        permit_join: int | None = None,
        rf_connected: bool | None = None,
        time_format: Literal["12h", "24h"] | None = None,
        time_zone: str | None = None,
        unlock: int | None = None,
        update_channel: Literal["alpha", "beta", "stable"] | None = None,
        utc: str | None = None,
        zigbee_channel: Literal[11, 15, 20, 25] | None = None,
        websocket_notify_all: bool | None = None,
    ) -> dict[str, Any]:
        """Modify configuration parameters.

        Supported values:
        - discovery [bool]
            Set gateway discovery over the internet active or inactive.
        - group_delay [int] 0-5000
            Time between two group commands in milliseconds.
        - light_last_seen_interval [int] 1-65535 default 60
            Sets the number of seconds where the timestamp for "lastseen"
            is updated at the earliest for light resources. For any such update,
            a seperate websocket event will be triggered.
        - name [str] 0-16 characters
            Name of the gateway.
        - network_open_duration [int] 1-65535
            Sets the lights and sensors search duration in seconds.
        - otau_active [bool]
            Set OTAU active or inactive
        - permit_join [int] 0-255
            Open the network so that other zigbee devices can join.
            0 = network closed
            255 = network open
            1–254 = time in seconds the network remains open
            The value will decrement automatically.
        - rf_connected [bool]
            Set to true to bring the Zigbee network up and false to bring it down.
            This has the same effect as using the Join and Leave buttons in deCONZ.
        - time_format [str] 12h|24h
            Can be used to store the timeformat permanently.
        - time_zone [str]
            Set the timezone of the gateway (only on Raspberry Pi).
        - unlock [int] 0-600 (seconds)
            Unlock the gateway so that apps can register themselves to the gateway.
        - update_channel [str] stable|alpha|beta
            Set update channel.
        - utc [str]
            Set the UTC time of the gateway (only on Raspbery Pi)
            in ISO 8601 format (yyyy-MM-ddTHH:mm:ss).
        - zigbee_channel [int] 11|15|20|25
            Set the zigbeechannel of the gateway.
            Notify other Zigbee devices also to change their channel.
        - websocket_notify_all [bool] default True
            When true all state changes will be signalled through the websocket connection.
        """
        data = {
            key: value
            for key, value in {
                "discovery": discovery,
                "groupdelay": group_delay,
                "lightlastseeninterval": light_last_seen_interval,
                "name": name,
                "networkopenduration": network_open_duration,
                "otauactive": otau_active,
                "permitjoin": permit_join,
                "rfconnected": rf_connected,
                "timeformat": time_format,
                "timezone": time_zone,
                "unlock": unlock,
                "updatechannel": update_channel,
                "utc": utc,
                "zigbeechannel": zigbee_channel,
                "websocketnotifyall": websocket_notify_all,
            }.items()
            if value is not None
        }
        return await self.request("put", path="/config", json=data)



"""Python library to connect deCONZ and Home Assistant to work together."""

from typing import Literal, TypedDict

from . import SensorBase


class TypedAirQualityState(TypedDict):
    """Air quality state type definition."""

    airquality: Literal[
        "excellent",
        "good",
        "moderate",
        "poor",
        "unhealthy",
        "out of scale",
    ]
    airqualityppb: int


class TypedAirQuality(TypedDict):
    """Air quality type definition."""

    state: TypedAirQualityState


class AirQuality(SensorBase):
    """Air quality sensor."""

    ZHATYPE = ("ZHAAirQuality",)

    raw: TypedAirQuality

    @property
    def air_quality(
        self,
    ) -> Literal["excellent", "good", "moderate", "poor", "unhealthy", "out of scale"]:
        """Air quality.

        Supported values:
        - "excellent"
        - "good"
        - "moderate"
        - "poor"
        - "unhealthy"
        - "out of scale"
        """
        return self.raw["state"]["airquality"]

    @property
    def air_quality_ppb(self) -> int:
        """Air quality PPB."""
        return self.raw["state"]["airqualityppb"]



"""Python library to connect deCONZ and Home Assistant to work together."""

from __future__ import annotations

from typing import TypedDict

from . import SensorBase


class TypedTemperatureState(TypedDict):
    """Temperature state type definition."""

    temperature: int


class TypedTemperature(TypedDict):
    """Temperature type definition."""

    state: TypedTemperatureState


class Temperature(SensorBase):
    """Temperature sensor."""

    ZHATYPE = ("ZHATemperature", "CLIPTemperature")

    raw: TypedTemperature

    @property
    def temperature(self) -> int:
        """Temperature."""
        return self.raw["state"]["temperature"]

    @property
    def scaled_temperature(self) -> float:
        """Scaled temperature."""
        return round(self.temperature / 100, 1)
